
dvardef(varname, vardefault, min, max, type)
{
	mapname = getdvar("mapname");
	gametype = getdvar("g_gametype");
	multigtmap = gametype + "_" + mapname;

	// check against gametype
	tempvar = varname + "_" + gametype;
	if(getdvar(tempvar) != "") varname = tempvar;

	// check against the map name
	tempvar = varname + "_" + mapname;
	if(getdvar(tempvar) != "") varname = tempvar;

	// check against the gametype and the map name
	tempvar = varname + "_" + multigtmap;
	if(getdvar(tempvar) != "") varname = tempvar;

	// get the variable's definition
	switch(type)
	{
		case "int":
		if(getdvar(varname) == "") definition = vardefault;
		else definition = getdvarint(varname);
		break;

		case "float":
		if(getdvar(varname) == "") definition = vardefault;
		else definition = getdvarfloat(varname);
		break;

		case "string":
		default:
		if(getdvar(varname) == "") definition = vardefault;
		else definition = getdvar(varname);
		break;
	}

	// check to see if the value is within the min & max parameters
	if(type != "string")
	{
		if(min != 0 && definition < min) definition = min;
		else if(max != 0 && definition > max) definition = max;
	}

	return definition;
}

setDvarDefault( dvarName, setVal, minVal, maxVal, type )
{
	// no value set
	if ( getDvar( dvarName ) != "" )
	{
		if ( isString( setVal ) )
			setVal = getDvar( dvarName );
		else
		{
			if (isDefined(type) && type == "float")
				setVal = getDvarFloat( dvarName );
			else
				setVal = getDvarInt( dvarName );
		}
	}
		
	if ( isDefined( minVal ) && !isString( setVal ) )
		setVal = max( setVal, minVal );

	if ( isDefined( maxVal ) && !isString( setVal ) )
		setVal = min( setVal, maxVal );

	setDvar( dvarName, setVal );
	return setVal;
}

isMoving(player)
{
	moving = true;

	if(isDefined(player.old_position))
		if(player.old_position == player.origin) moving = false;

	player.old_position = player.origin;

	return moving;
}


monotone( str )
{
	if ( !isDefined( str ) || ( str == "" ) )
		return ( "" );

	_s = "";

	_colorCheck = false;
	for ( i = 0; i < str.size; i++ )
	{
		ch = str[ i ];
		if ( _colorCheck )
		{
			_colorCheck = false;

			switch ( ch )
			{
			  case "0":	// black
			  case "1":	// red
			  case "2":	// green
			  case "3":	// yellow
			  case "4":	// blue
			  case "5":	// cyan
			  case "6":	// pink
			  case "7":	// white
			  case "8":	// Olive
			  case "9":	// Grey
			  	break;
			  default:
			  	_s += ( "^" + ch );
			  	break;
			}
		}
		else if ( ch == "^" )
			_colorCheck = true;
		else
			_s += ch;
	}
	return ( _s );
}

//lowercase(str)
//{
//	return(convertChar(str, "U-L" ));
//}

//uppercase(str)
//{
//	return(convertChar(str, "L-U" ));
//}

bounceObject(vRotation, vVelocity, vOffset, angles, radius, falloff, bouncesound, bouncefx, objecttype)
{
	level endon("game_ended");
	self endon("disconnect");
	self endon("ex_bounceobject");

	if(!isDefined(objecttype)) return;

	self thread putinQ(objecttype);

	// Hide until everthing is setup
	self hide();

	// Setup default values
	if(!isDefined(vRotation))	vRotation = (0,0,0);
	pitch = vRotation[0]*0.05;	// Pitch/frame
	yaw = vRotation[1]*0.05;	// Yaw/frame
	roll = vRotation[2]*0.05;		// Roll/frame

	if(!isDefined(vVelocity))	vVelocity = (0,0,0);
	if(!isDefined(vOffset)) vOffset = (0,0,0);
	if(!isDefined(falloff)) falloff = 0.5;

	// Spawn anchor (the object that we will rotate)
	self.bounce_anchor = spawn("script_origin", self.origin );
	self.bounce_anchor.angles = self.angles;

	// Link to anchor
	self linkto( self.bounce_anchor, "", vOffset, angles );
	self show();

	wait .05;	// Let it happen

	if(isDefined(level.ex_gravity)) gravity = level.ex_gravity;
	else gravity = 100;

	// Set gravity
	vGravity = (0,0,-0.02 * gravity);

	stopme = 0;
	notrace = 5;	// Avoid bullettrace for the first number of frames

	// Drop with gravity
	for(;;)
	{
		// Let gravity do, what gravity do best
		vVelocity +=vGravity;

		if(!isDefined(self.bounce_anchor)) return;

		// Get destination origin
		neworigin = self.bounce_anchor.origin + vVelocity;

		// Check for impact, check for entities but not myself.
		if(!notrace)
		{
			trace=bulletTrace(self.bounce_anchor.origin,neworigin,false,undefined); 
			if(trace["fraction"] != 1)	// Hit something
			{
				// Place object at impact point - radius
				distance = distance(self.bounce_anchor.origin,trace["position"]);
				if(distance)
				{
					fraction = (distance - radius) / distance;
					delta = trace["position"] - self.bounce_anchor.origin;
					delta2 = common_scripts\utility::vectorScale(delta,fraction);
					neworigin = self.bounce_anchor.origin + delta2;
				}
				else neworigin = self.bounce_anchor.origin;

				// Play sound if defined
				if(isDefined(bouncesound)) self.bounce_anchor playSound(bouncesound + trace["surfacetype"]);	

				// Test if we are hitting ground and if it's time to stop bouncing
				if(vVelocity[2] <= 0 && vVelocity[2] > -10) stopme++;
				if(stopme==5)
				{
					stopme=0;
					// Set origin to impactpoint	
					self.bounce_anchor.origin = neworigin;
					wait .05;
					// Delete anchor to save gamestate size
					self unlink();
					self.bounce_anchor delete();
					return;
				}
				// Play effect if defined and it's a hard hit
				if(isDefined(bouncefx) && length(vVelocity) > 20) playfx(bouncefx,trace["position"]);

				// Decrease speed for each bounce.
				vSpeed = length(vVelocity) * falloff;

				// Calculate new direction (Thanks to Hellspawn this is finally done correctly)
				vNormal = trace["normal"];
				vDir = common_scripts\utility::vectorScale(vectorNormalize( vVelocity ),-1);
				vNewDir = ( common_scripts\utility::vectorScale(common_scripts\utility::vectorScale(vNormal,2),vectorDot( vDir, vNormal )) ) - vDir;

				// Scale vector
				vVelocity = common_scripts\utility::vectorScale(vNewDir, vSpeed);
	
				// Add a small random distortion
				vVelocity += (randomFloat(1)-0.5,randomFloat(1)-0.5,randomFloat(1)-0.5);
			}
		}
		else notrace--;

		if(!isDefined(self.bounce_anchor)) return;

		self.bounce_anchor.origin = neworigin;

		// Rotate pitch
		a0 = self.bounce_anchor.angles[0] + pitch;
		while(a0<0) a0 += 360;
		while(a0>359) a0 -=360;

		// Rotate yaw
		a1 = self.bounce_anchor.angles[1] + yaw;
		while(a1<0) a1 += 360;
		while(a1>359) a1 -=360;

		// Rotate roll
		a2 = self.bounce_anchor.angles[2] + roll;
		while(a2<0) a2 += 360;
		while(a2>359) a2 -=360;
		self.bounce_anchor.angles = (a0,a1,a2);
		
		// Wait one frame
		wait .05;
	}
}

putinQ(type)
{
	if(!isDefined(type)) self notify("ex_bounceobject");
	else
	{
		index = level.ex_objectQcurrent[type];
	
		level.ex_objectQcurrent[type]++;
	
		if(level.ex_objectQcurrent[type] >= level.ex_objectQsize[type]) level.ex_objectQcurrent[type] = 0;
	
		if(isDefined(level.ex_objectQ[type][index]))
		{
			level.ex_objectQ[type][index] notify("ex_bounceobject");
			wait .05; //Let thread die
	
			if(isDefined(level.ex_objectQ[type][index].anchor))
			{
				level.ex_objectQ[type][index] unlink();
				level.ex_objectQ[type][index].anchor delete();
			}
	
			if(isDefined(level.ex_objectQ[type][index])) level.ex_objectQ[type][index] delete();
		}
		
		level.ex_objectQ[type][index] = self;
	}
}

// The objectives array is shared between camper and ammocrates code!
getObjective()
{
	if(!isDefined(level.ex_objectives)) createObjectivesArray();

	objnum = 0;
	// Check slots 15 - 4
	for(i = 15; i >= 4; i--)
	{
		if(level.ex_objectives[i] == 0)
		{
			level.ex_objectives[i] = 1;
			objnum = i;
			break;
		}
	}
	return objnum;
}

deleteObjective(objnum)
{
	if(!isDefined(level.ex_objectives)) createObjectivesArray();

	if(level.ex_objectives[objnum] == 1)
	{
		objective_delete(objnum);
		level.ex_objectives[objnum] = 0;
	}
}

createObjectivesArray()
{
	if(!isDefined(level.ex_objectives)) level.ex_objectives = [];

	for(i = 0; i <= 15; i++)
	{
		if(i < 4) level.ex_objectives[i] = 1; // First 4 objectives are reserved
		  else level.ex_objectives[i] = 0;
	}
}
